//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  dualplot.cpp
//! \brief Implements class CPlot.

#include <iostream>

#include "../trivia/string_convs.hpp"
#include "dualplot.hpp"
#include "gnuplotter.hpp"
#include "ps_plotter.hpp"

#define S(a) triv::strg((a))

using std::string;
using std::vector;
using std::cout;

//! Constructor for plot window: setup for gnuplot and postscript.

CPlot::CPlot(int _iPlot, bool _logx, bool _logy, bool _p2D, bool _logz)
    : iPlot(_iPlot)
    , p2D(_p2D)
    , X("x", _logx)
    , Y("y", _logy)
    , Z("z", _logz)
    , maxpoints(20000)
    , with_errors(true)
    , equipoints(120)
    , refine(true)
    , gnuPlotter(new CGnuPlotter(iPlot))
    , ps_Plotter(new CPS_Plotter)
{}


//! Change one setup parameter per command.

void CPlot::set_aux(const string& cmd)
{
    if (cmd == "gxl") {
        X.set_log(!X.logflag);
        printf("set x %s\n", X.logflag ? "log" : "lin");
    } else if (cmd == "gxl+")
        X.set_log(true);
    else if (cmd == "gxl-")
        X.set_log(false);

    else if (cmd == "gyl") {
        Y.set_log(!Y.logflag);
        printf("set y %s\n", Y.logflag ? "log" : "lin");
    } else if (cmd == "gyl+")
        Y.set_log(true);
    else if (cmd == "gyl-")
        Y.set_log(false);

    else if (cmd == "gzl") {
        Z.set_log(!Z.logflag);
        printf("set z %s\n", Z.logflag ? "log" : "lin");
    } else if (cmd == "gzl+")
        Z.set_log(true);
    else if (cmd == "gzl-")
        Z.set_log(false);

    else if (cmd == "gxf") {
        X.force = !X.force;
        printf("force x %s\n", X.force ? "on" : "off");
    } else if (cmd == "gxf+")
        X.force = true;
    else if (cmd == "gxf-")
        X.force = false;
    else if (cmd == "gyf") {
        Y.force = !Y.force;
        printf("force y %s\n", Y.force ? "on" : "off");
    } else if (cmd == "gyf+")
        Y.force = true;
    else if (cmd == "gyf-")
        Y.force = false;

    else if (cmd == "ge")
        with_errors = !with_errors;
    else if (cmd == "ge+")
        with_errors = true;
    else if (cmd == "ge-")
        with_errors = false;

    else if (cmd == "gt")
        p2D = !p2D;
    else if (cmd == "gt+")
        p2D = true;
    else if (cmd == "gt-")
        p2D = false;

    else
        throw "unknown command " + cmd;
}


//! Clear 1D plot canvas, and plot coordinate frame (axes, ticks, labels).

void CPlot::start_frame1D(const string& caller, const string& xlabel, const string& ylabel)
{
    gnuPlotter->start_frame(X, Y);
    ps_Plotter->start_frame1D(caller, X, Y, xlabel, ylabel);
}


//! Clear 2D plot canvas, and print coordinate ranges

void CPlot::start_frame2D(const string& caller)
{
    ps_Plotter->start_frame2D(caller, X, Z, Y);
}


//! Plot coordinate frame (axes, ticks, labels).

void CPlot::close_frame2D(const string& xlabel, const string& zlabel, const string& ylabel)
{
    ps_Plotter->close_frame2D(X, Z, Y, xlabel, zlabel, ylabel);
}


//! Plot one spectrum.

void CPlot::add_spec(
    bool as_line, bool new_slice, int style_no, const vector<double>& xp, const vector<double>& yp,
    const vector<double>& dyp, const vector<string>& zentries,
    const string& xco, const string& yco, const string& info)
{
    gnuPlotter->add_spec(
        as_line, new_slice, with_errors,
        style_no, xp, yp, dyp, zentries, xco, yco, info);
    ps_Plotter->add_spec(
        as_line, new_slice,
        style_no, xp, yp, dyp, zentries, xco, yco, info);
}

//! Plot one bin of 2D plot.
void CPlot::ps_line(const string& line)
{
    ps_Plotter->main_line(line);
}

void CPlot::show_specs()
{
    gnuPlotter->show_specs();
}

//! Add documentation line to postscript output.

void CPlot::doc_TxLine(const string& line)
{
    ps_Plotter->doc_line("  {(" + line + ")} TxLine");
}


//! Add documentation line explaining a plot symbol to postscript output.

void CPlot::doc_PtTxLine(const string& line, int num)
{
    ps_Plotter->doc_line("  " + S(num) + " {(" + line + ")} PtTxLine");
}


//! Add documentation line explaining a curve style to postscript output.

void CPlot::doc_CvTxLine(const string& line, int num)
{
    ps_Plotter->doc_line("  " + S(num) + " {(" + line + ")} CvTxLine");
}


//! Write buffered plot to postscript file.

void CPlot::write_postscript(
    const string& mode, const string& fname,
    const string& ps_setup_dir, const string& ps_definitions)
{
    ps_Plotter->copy_header(mode, fname, ps_setup_dir, ps_definitions);
    ps_Plotter->write_data(fname);
}

//! Info line to characterize this plot window.

string CPlot::info() const
{
    if ( p2D )
        return "2D  x: " + X.info() +  "  z: " + Z.info() + "  y: " + Y.info();
    return "1D  x: " + X.info() +  "  y: " + Y.info();
}


//! Send one line to gnuplot fifo.

void CPlot::gp_write(const string& in)
{
    gnuPlotter->write(in);
}
